#ifndef MATRICE__HPP
#define MATRICE__HPP

#include <cstdlib>
#include <iostream>
#include <cmath>
using namespace std;

typedef unsigned int u_int;

class Matrice {
	public:
		// ------------------------------------------------ COMUNE ---------------------------------------------------

		Matrice(u_int sx, u_int sy, double v = 0.): _sx(sx), _sy(sy) {				// OBB A/B
		
			_m = new double*[_sx];
			for(u_int x = _sx; x--; ) {
				_m[x] = new double[_sy];
				for(u_int y = _sy; y--; ) _m[x][y] = v;
			}

		};

		Matrice(const Matrice &m): _sx(m._sx), _sy(m._sy) {					// OBB A/B

			_m = new double*[_sx];
			for(u_int x = _sx; x--; ) _m[x] = new double[_sy];
			*this = m;
		
		};

		~Matrice() {										// OBB A/B
			
			for(u_int x = _sx; x--; ) delete[] _m[x];
			delete[] _m;
		
		};

		u_int SX() const { return _sx; };							// OBB A/B
		u_int SY() const { return _sy; };							// OBB A/B

		friend const Matrice &Randomize(Matrice &m) {						// FAC A/B

			for(u_int x = m._sx; x--; ) for(u_int y = m._sy; y--; ) m._m[x][y] = drand48();
			return m;
		
		};

		double  operator()(u_int x, u_int y) const { return _m[x][y]; };			// OBB A/B
		double &operator()(u_int x, u_int y)       { return _m[x][y]; };			// OBB A/B

		const Matrice &operator=(const Matrice &m) {						// OBB A/B

			for(u_int x = _sx; x--; ) for(u_int y = _sy; y--; ) _m[x][y] = m._m[x][y];
			return *this;

		};

		Matrice operator*(double a) const {
		
			Matrice out(*this);
			for(u_int x = _sx; x--; ) for(u_int y = _sy; y--; ) out._m[x][y] *= a;
		
			return out;
		};

		const Matrice &operator+=(const Matrice &m) {						// FAC A, OBB B
		
			for(u_int x = _sx; x--; ) for(u_int y = _sy; y--; ) _m[x][y] += m._m[x][y];
			return *this;

		};
		Matrice operator+(const Matrice &m) const { return Matrice(*this) += m; };		// FAC A, OBB B

		const Matrice &operator-=(const Matrice &m) {						// FAC A/B
		
			for(u_int x = _sx; x--; ) for(u_int y = _sy; y--; ) _m[x][y] -= m._m[x][y];
			return *this;

		};
		Matrice operator-(const Matrice &m) const { return Matrice(*this) -= m; };		// FAC A/B

		Matrice operator*(const Matrice &m) const {						// OBB A, FAC B

			Matrice out(_sx, m._sy, 0.);
			for(u_int x = _sx; x--; ) for(u_int y = m._sy; y--; ) 
				for(u_int k = _sy; k--; ) out._m[x][y] += _m[x][k] * m._m[k][y];

		
			return out;
		};

		const Matrice &operator+() const { return *this; };					// FAC A/B	
		Matrice operator-() const {								// FAC A/B
		
			Matrice out(*this);
			for(u_int x = _sx; x--; ) for(u_int y = _sy; y--; ) out._m[x][y] *= -1;
		
			return out;
		};

		bool operator==(const Matrice &m) const {						// FAC A/B
		
			if(_sx != m._sx || _sy != m._sy) return false;
			for(u_int x = _sx; x--; ) for(u_int y = _sy; y--; ) if(_m[x][y] != m._m[x][y]) return false;
		
			return true;
		};
		bool operator!=(const Matrice &m) const { return !((*this) == m); };			// FAC A/B

		friend Matrice Trasposta(const Matrice &m) {						// FAC A/B
		
			Matrice out(m._sy, m._sx);
			for(u_int x = m._sx; x--; ) for(u_int y = m._sy; y--; ) out._m[y][x] = m._m[x][y];
		
			return out;
		};

		// -----------------------------------------------------------------------------------------------------------

		// ---------------- p-Norma --------------------
		Matrice operator^(u_int k) const {							// OBB A

			Matrice out(*this);
			while(--k) out = out * (*this);
		
			return out;
		};

		friend double Tr(const Matrice &m) {							// OBB A
		
			double tr = 0.;
			for(u_int x = (m._sx < m._sy)?m._sx:m._sy; x--; ) tr += m._m[x][x];
		
			return tr;
		};

		friend Matrice Abs(const Matrice &m) {

			Matrice out(m);
			for(u_int x = out._sx; x--; ) for(u_int y = out._sy; y--; ) out._m[x][y] = fabs(out._m[x][y]);

			return out;	
		};

		friend double Norma(const Matrice &m, u_int p) {					// FAC A
		
			if(p == 0) return 0;
			else return pow(Tr(Abs(m)^p), 1./p);
		
		};
		// ----------------------------------------------

		// ------------- LAPLACE Method -----------------
		Matrice Minore(u_int xx, u_int yy) const {						// OBB B
		
			Matrice minore(_sx-1, _sx-1);
			for(u_int x = _sx; x--; ) if(x != xx) 
				for(u_int y = _sy; y--; ) if(y != yy) minore._m[x - (x > xx)][y - (y > yy)] = _m[x][y];

			return minore;
		};

		friend double Det(const Matrice &m) {							// FAC A
		
			if(m._sx == 1) return m._m[0][0];

			double det = 0.;
			for(u_int y = m._sy; y--; ) 
				if((y % 2) == 0) det += m._m[0][y] * Det(m.Minore(0, y));
				else             det -= m._m[0][y] * Det(m.Minore(0, y));
		
			return det;
		};
		// ----------------------------------------------

		       ostream &operator<<(ostream &out)                   const { return   _Print(out); };		// FAC
		friend ostream &operator<<(ostream &out, const Matrice &m)       { return m._Print(out); };		// FAC
	private:
		const u_int _sx, _sy;	
		double **_m;

		ostream &_Print(ostream &out) const {									// FAC
		
			for(u_int x = 0; x < _sx; ++x) {
				for(u_int y = 0; y < _sy; ++y) out << _m[x][y] << " ";
				out << endl;
			}
		
			return out;
		};
};

class Vettore : public Matrice {											// FAC
	public:
		Vettore(u_int sx): Matrice(sx, 1) {};
	private:
};

#endif
